
/**
 * Format a date
 * @param {Date} date - A Javascript Date object
 * @param {String} format - The format of the outputted date
 * @returns {String} - The formatted date
 */
(function(){
	"use strict";
	ProtoDate.prototype.format = function(format) {
		if (!ProtoDate.isDate(this)) return false;
		var buffer = []; 
		for(var i=0; i<format.length; i++){
			switch(format[i]){
				// If the current char is a "\" then skip it and add then next literal char
				case "\\": buffer.push(format[++i]); break;

				// Symbols that represent numbers
				case "Y": buffer.push("" + this.getFullYear()); break;
				case "y": buffer.push(("" + this.getFullYear()).substring(2)); break;
				case "m": buffer.push(("0" + (this.getMonth() + 1)).substr(-2, 2)); break;
				case "n": buffer.push("" + (this.getMonth() + 1)); break;
				case "t": buffer.push("" + new ProtoDate(this.getFullYear(), this.getMonth() + 1, 0).getDate()); break; 
				case "d": buffer.push(("0" + this.getDate()).substr(-2, 2)); break;
				case "j": buffer.push(this.getDate() + ""); break;
				case "w": buffer.push(this.getDay()); break;
				case "g": buffer.push("" + (this.getHours() > 12 ? this.getHours() - 12 : this.getHours())); break;
				case "G": buffer.push("" + (this.getHours())); break;
				case "h": buffer.push(("0" + (this.getHours() > 12 ? this.getHours() - 12 : this.getHours())).substr(-2, 2)); break;
				case "H": buffer.push(("0" + (this.getHours()+"")).substr(-2, 2)); break;
				case "i": buffer.push(("0" + this.getMinutes()).substr(-2, 2)); break;
				case "s": buffer.push(("0" + this.getSeconds()).substr(-2, 2)); break;
				case "N": buffer.push(this.getDay()==0?7:this.getDay()); break;
				case "L": buffer.push((this.getFullYear() % 4 == 0 && this.getFullYear() % 100 != 0) || this.getFullYear() % 400 == 0 ? "1" : "0"); break;
				case "o": buffer.push(this.getMonth()==0&&this.getDate()<6&&this.getDay()<4?this.getFullYear()-1:this.getFullYear()); break;
				case "B": buffer.push(Math.floor((((this.getUTCHours() + 1) % 24) + this.getUTCMinutes() / 60 + this.getUTCSeconds() / 3600) * 1000 / 24)); break;
				case "v": buffer.push((this.getTime()+"").substr(-3)); break;
				case "Z": buffer.push(this.getTimezoneOffset()*60); break;
				case "U": buffer.push(Math.floor(this.getTime()/1000)); break;

				// Symbols that represent text
				case "a": buffer.push(this.getHours() > 11 ? "pm" : "am"); break;
				case "A": buffer.push(this.getHours() > 11 ? "PM" : "AM"); break;
				case "l": buffer.push(ProtoDate.DAYS[this.getDay()]); break;
				case "D": buffer.push(ProtoDate.DAYS[this.getDay()].substr(0, 3)); break;
				case "F": buffer.push(ProtoDate.MONTHS[this.getMonth()]); break;
				case "M": buffer.push(ProtoDate.MONTHS[this.getMonth()].substring(0, 3)); break;
				case "c": buffer.push(this.toISOString()); break;

				// Ordinal suffix
				case "S":
					var suffix = false;
					var ones = buffer[buffer.length-1].toString()[1];
					var tens = buffer[buffer.length-1].toString()[0];
					if(undefined === ones){
					  ones = tens;
					  tens = null;
					} 
					if(ones == "1") suffix = "st";
					if(ones == "2") suffix = "nd";
					if(ones == "3") suffix = "rd";
					if(tens == "1" || !suffix) suffix = "th";
					buffer.push(suffix);
					break;

				// ISO-8601 Week number
				case "W":
					var startDate = new ProtoDate(this.getFullYear(), 0);
					var endDate = new ProtoDate(this.getFullYear(), this.getMonth(), this.getDate());
					while(endDate.getDay() < 6) endDate.setDate(endDate.getDate()+1);
					endDate = endDate.getTime();
					var weekNo = 0;
					while(startDate.getTime() < endDate){
						if(startDate.getDay() == 4) weekNo++;
						startDate.setDate(startDate.getDate()+1);
					}
					buffer.push(weekNo);
					break;

				// Day of the year
				case "z":
					var startDate = new ProtoDate(this.getFullYear(), 0, 1, 0, 0, 0, 0);
					var dayNo = 0;
					while(startDate.getTime() < this.getTime()){
						dayNo++;
						startDate.setDate(startDate.getDate()+1);
					}
					buffer.push(dayNo);
					break;
				
				// Julian Day
				case "J":
					var y = parseInt(this.format("Y"));
					var m = parseInt(this.format("n"));
					if(m === 1 || m === 2){
						y -= 1;
						m += 12;
					}
					var d = parseInt(this.format("j"));
					var a = Math.floor(y/100);
					var b = Math.floor(a/4);
					var c = 2-a+b;
					var e = Math.floor(365.25*(y+4716));
					var f = Math.floor(30.6001*(m+1));
					buffer.push(c+d+e+f-1524.5);
					break;
					
				// Moon Phase
				case "P":
					var jd = this.format("J");
					var dsn = jd - 2451549.5;
					var nm = dsn / 29.53;
					var dic = parseFloat("0."+((nm+"").split(".")[1])) * 29.53;
					if(dic > 27.65625) return "New Moon";
					if(dic > 23.96875) return "Waning Crescent";
					if(dic > 20.28125) return "Third Quarter";
					if(dic > 16.59375) return "Waning Gibbous";
					if(dic > 12.90625) return "Full Moon";
					if(dic > 9.21875) return "Waxing Gibbous";
					if(dic > 5.53125) return "First Quarter";
					if(dic > 1.84375) return "Waxing Crescent";
					return "New Moon";
					break;
				
				default: buffer.push(format[i]); break;
			}
		}
		return buffer.join('');
	};
})();